% conv_2l: convergence tests of slab.c for extreme 2-layer cases
%
% Figures for sla paper.
%
% barnett 4/20/03

hom=0;
% -------------------------------------------------------------
% systems...  2-layers, rows give system choices.
dz = 8;
%2 mild 2-layer cases...
%musps = [0.8 1.5; 1.5 0.8];
%muas = [0.01 0.015; 0.015 0.01];

%4 extreme 2-layer cases...
musps = [0.3 3; 3 0.3; 0.3 3; 3 0.3];
muas = [0.003 0.03; 0.003 0.03; 0.03 0.003; 0.03 0.003];

%1 homog case...
%hom = 1;   % makes test relative to homog analytic soln.
%musps = [1.0 1.0];
%muas = [0.01 0.01];

n = 1.4;
bcs = 1;

% meas...
tmin = 100;
tmax = 5000;
dtgate = 15.7;   % avoid commensurate with dt
R = 40; %[0 10 20 30 40 50];
 
% error measure params... gamma, epsilon
g = 1e5;
e = 1e-2;

% set up resampled t array for matlab display...
ntgate=(tmax-tmin)/dtgate + 1;
sigt = tmin + dtgate*(0:ntgate-1)';
nt = length(sigt);

if ~exist('CONV') 
    CONV = input('CONV (0=dt/t, 1=dt_kill, 2=h, 3=dt(const)) > ');  
end

switch CONV
  
 case 0
  % -------------------------------------------------------------------------
  % test beta=dt/t convergence of 3d signal, on fixed 1d system.

  h = 0.5;
  %bs = [0.2 0.14 0.1 0.07 0.05 0.035 0.025 0.018 0.0125 0.009 0.00625];
  bs = [0.2 0.14 0.1 0.07 0.05 0.035 0.025 0.018 0.0125];
  scheme = 1;
  
  nb = length(bs);
  sigb = zeros(nt, nb);
  ns = size(musps,1);  % modify
  timb = zeros(ns, nb);

  
% loop over systems...
for is = 1:ns
    
  musp = musps(is,:);
  mua = muas(is,:);
  for ib = 1:nb
    tic;
    sigb(:,ib) = slab(tmin,tmax,dtgate,R,mua,musp,dz,n,...
		      [1e-6,-h,-bs(ib),1e-6,1e-12,-1,129,scheme,bcs]); %129
    timb(is,ib) = toc;
  end

  % sige = sigb(:,nb); % `exact' signal.
  if ~hom
    tic;
    %sige = slab(tmin,tmax,dtgate,R,mua,musp,dz,n,...
    %		      [1e-6,-h,-0.0,1e-6,1e-12,-1,129,2,bcs]); % FE scheme.
    sige = slab(tmin,tmax,dtgate,R,mua,musp,dz,n,...
		[1e-6,-h,-0.5,1e-6,1e-12,-1,129,3,bcs]); % CN const.
    toc
  else
    % only for homog...
    sige = slab_homog_inf(tmin,tmax,dtgate,R,mua(1),musp(1),[],n,...
			  [0,0,0,0,0,0,0,0,bcs]);
  end

  % compute error baseline (errb)...
  peak = max(sige);
  errb = e*sige;  % fixed relative error size e.
  % poisson-noise type cutoff of dyn range...
  errb = max((peak/g)*sqrt(g*sige/peak), errb);
  errb = max(peak/g, errb); %   hard dynamic range cutoff
  nbe = nb;   % # h choices for err plot

  figure;
  subplot(2,1,1);
  semilogy(repmat(sigt,[1 nb]),abs(sigb),'-');
  legnum(bs, 2, 'beta=');
  hold on; semilogy(sigt, abs(sige), '--'); hold off;
  hold on; semilogy(sigt, errb, ':'); hold off;
  title(sprintf('conv 2l 0 SYS%d: dz=%g, musp=(%g,%g) mua=(%g, %g)', ...
  is, dz, musp, mua));
  
  subplot(2,1,2);
  semilogy(repmat(sigt,[1 nbe]),abs(sigb(:,1:nbe)-repmat(sige,[1 ...
  nbe]))./repmat(errb,[1 nbe]),'-');
  % test power laws in beta: goes from 2 to 1...
  % semilogy(repmat(sigt,[1 nbe]),abs(sigb(:,1:nbe)-repmat(sige,[1 ...
  % nbe]))./kron(errb,bs(1:nbe)),'-');
  hold on; semilogy([tmin tmax], [1 1], ':'); hold off;
  legnum(bs(1:nbe), 2, 'beta=');
  title(sprintf('conv 2l 0 SYS%d: errors (exact:beta=%g), rel to e=%g, g=%g',...
  is, bs(nb), e, g));

end

if 0
figure;
loglog(repmat(bs,[ns 1])', timb', '+-');
axis tight;
xlabel('beta=dt/t');
ylabel('runtime (s)');
legnum(1:ns, 1, 'SYS');
title('conv 2l 0: all timings');
else
  timb
end

  
 case 1
  
  
  
  
  
  
 case 2
  % -------------------------------------------------------------------------
  
% set of h
%hs = [3.2 2.2 1.5 1 0.68];
%hs = [2 1.4 1.0 0.7 0.5 0.35 0.25 0.18 0.125 0.09 0.0625];   % for mild ones
hs = [2 1.0 0.5 0.25 0.125 0.0625];   % for mild ones
%hs = [2 1.0 0.5 0.25];   % for mild ones
%hs = [2 1.4 1.0 0.7 0.5 0.35];   % for extreme systems
err_dt = 0.01;  % not used by FE, which used courant condition.
scheme = 1;

%--------------------------------------------------------------

nh = length(hs);
sigh = zeros(nt, nh);
ns = size(musps,1);  % modify
timh = zeros(ns, nh);

% loop over systems...
for is = 1:ns
    
  musp = musps(is,:);
  mua = muas(is,:);
  for ih = 1:nh
    tic;
    sigh(:,ih) = slab(tmin,tmax,dtgate,R,mua,musp,dz,n,...
		      [1e-6,-hs(ih),err_dt,1e-6,1e-12,-1,129,scheme,bcs]); %129
    timh(is,ih) = toc;
  end

  sige = sigh(:,nh); % `exact' signal.
  % compute error baseline...
  peak = max(sige);
  errb = max(peak/g, e*sige);
  nhe = nh-2;   % # h choices for err plot

  figure;
  subplot(2,1,1);
  semilogy(repmat(sigt,[1 nh]),sigh,'-');
  legnum(hs, 2, 'h=');
  hold on; semilogy(sigt, errb, ':'); hold off;
  title(sprintf('conv 2l SYS%d: dz=%g, musp=(%g,%g) mua=(%g, %g)', ...
		 is, dz, musp, mua));
  
  subplot(2,1,2);
  semilogy(repmat(sigt,[1 nhe]),abs(sigh(:,1:nhe)-repmat(sige,[1 ...
		    nhe]))./repmat(errb,[1 nhe]),'-');
  hold on; semilogy([tmin tmax], [1 1], ':'); hold off;
  legnum(hs(1:nhe), 2, 'h=');
  title(sprintf('conv 2l SYS%d: errors (exact:h=%g), rel to e=%g, g=%g',...
		 is, hs(nh), e, g));

end


 case 3
  % 333333333333333333333333333333333333333333333333333333333333333333333333333
  % test dt convergence of 3d signal, on fixed 1d system.

  h = 0.25;
  %bs = [10 7 5 3.5 2.5 1.8 1.25];
  bs = [20 10 5 2.5 1.25];
  be = 0.25;        % exact
  scheme = 3;
  
  nb = length(bs);
  sigb = zeros(nt, nb);
  ns = size(musps,1);  % modify
  timb = zeros(ns, nb);

  
% loop over systems...
for is = 1:ns
    
  musp = musps(is,:);
  mua = muas(is,:);
  for ib = 1:nb
    tic;
    sigb(:,ib) = slab(tmin,tmax,dtgate,R,mua,musp,dz,n,...
		      [1e-6,-h,-bs(ib),1e-6,1e-12,-1,129,scheme,bcs]); %129
    timb(is,ib) = toc;
  end

  % sige = sigb(:,nb); % `exact' signal.
  if ~hom
    tic;
    sige = slab(tmin,tmax,dtgate,R,mua,musp,dz,n,...
		[1e-6,-h,-be,1e-6,1e-12,-1,129,scheme,bcs]); % CN-const
    toc
  else
    % only for homog...
    sige = slab_homog_inf(tmin,tmax,dtgate,R,mua(1),musp(1),[],n,...
			  [0,0,0,0,0,0,0,0,bcs]);
  end
  
  % compute error baseline (errb)...
  peak = max(sige);
  errb = e*sige;  % fixed relative error size e.
  % poisson-noise type cutoff of dyn range...
  errb = max((peak/g)*sqrt(g*sige/peak), errb);
  errb = max(peak/g, errb); %   hard dynamic range cutoff
  nbe = nb;   % # h choices for err plot

  figure;
  subplot(2,1,1);
  semilogy(repmat(sigt,[1 nb]),abs(sigb),'-');
  legnum(bs, 2, 'dt=');
  hold on; semilogy(sigt, sige, '--'); hold off;
  hold on; semilogy(sigt, errb, ':'); hold off;
  title(sprintf('conv 2l 0 SYS%d: dz=%g, musp=(%g,%g) mua=(%g, %g)', ...
  is, dz, musp, mua));
  
  subplot(2,1,2);
  semilogy(repmat(sigt,[1 nbe]),abs(sigb(:,1:nbe)-repmat(sige,[1 ...
  nbe]))./repmat(errb,[1 nbe]),'-');
  % test power laws in beta: goes from 2 to 1...
  % semilogy(repmat(sigt,[1 nbe]),abs(sigb(:,1:nbe)-repmat(sige,[1 ...
  % nbe]))./kron(errb,bs(1:nbe)),'-');
  hold on; semilogy([tmin tmax], [1 1], ':'); hold off;
  legnum(bs(1:nbe), 2, 'd=');
  title(sprintf('conv 2l 0 SYS%d: errors (exact:dt=%g), rel to e=%g, g=%g',...
  is, be, e, g));

end

 case 4
  % 444444444444444444444444444444444444444444444444444444444444444444444444444
  % test dt convergence of 3d signal, on fixed 1d system, const-var CN

  h = 0.25;
  bs = [10 7 5 3.5 2.5 1.8 1.25];
  be = 0.25;        % exact
  scheme = 4;
  t0 = 400;
  beta = 0.05;
  
  nb = length(bs);
  sigb = zeros(nt, nb);
  ns = size(musps,1);  % modify
  timb = zeros(ns, nb);

  
% loop over systems...
for is = 1:ns
    
  musp = musps(is,:);
  mua = muas(is,:);
  for ib = 1:nb
    tic;
    sigb(:,ib) = slab(tmin,tmax,dtgate,R,mua,musp,dz,n,...
		      [1e-6,-h,-bs(ib),1e-6,1e-12,-1,129,scheme,bcs,t0,beta]);
    timb(is,ib) = toc;
  end

  % sige = sigb(:,nb); % `exact' signal.
  if ~hom
    tic;
    sige = slab(tmin,tmax,dtgate,R,mua,musp,dz,n,...
		[1e-6,-h,-be,1e-6,1e-12,-1,129,3,bcs]); % CN-const
    toc
  else
    % only for homog...
    sige = slab_homog_inf(tmin,tmax,dtgate,R,mua(1),musp(1),[],n,...
			  [0,0,0,0,0,0,0,0,bcs]);
  end
  
  % compute error baseline (errb)...
  peak = max(sige);
  errb = e*sige;  % fixed relative error size e.
  % poisson-noise type cutoff of dyn range...
  errb = max((peak/g)*sqrt(g*sige/peak), errb);
  errb = max(peak/g, errb); %   hard dynamic range cutoff
  nbe = nb;   % # h choices for err plot

  figure;
  subplot(2,1,1);
  semilogy(repmat(sigt,[1 nb]),abs(sigb),'-');
  legnum(bs, 2, 'dt=');
  hold on; semilogy(sigt, abs(sige), '--'); hold off;
  hold on; semilogy(sigt, errb, ':'); hold off;
  title(sprintf('conv 2l 0 SYS%d: dz=%g, musp=(%g,%g) mua=(%g, %g)', ...
  is, dz, musp, mua));
  
  subplot(2,1,2);
  semilogy(repmat(sigt,[1 nbe]),abs(sigb(:,1:nbe)-repmat(sige,[1 ...
  nbe]))./repmat(errb,[1 nbe]),'-');
  % test power laws in beta: goes from 2 to 1...
  % semilogy(repmat(sigt,[1 nbe]),abs(sigb(:,1:nbe)-repmat(sige,[1 ...
  % nbe]))./kron(errb,bs(1:nbe)),'-');
  hold on; semilogy([tmin tmax], [1 1], ':'); hold off;
  legnum(bs(1:nbe), 2, 'd=');
  title(sprintf('conv 2l 0 SYS%d: errors (exact:dt=%g), rel to e=%g, g=%g',...
  is, be, e, g));

end

end

% switch done -----------------------------------------------
if 0
figure;
loglog(repmat(hs,[ns 1])', timh', '+-');
axis tight;
xlabel('h (mm)');
ylabel('runtime (s)');
legnum(1:ns, 1, 'SYS');
title('conv 2l: all timings');
end



% end
