% SLA: box convergence test for 2-layer system.


% don't forget to set mu_shift fixed for e_L convergence !


% 1 homog then 4 extreme 2-layer cases...
musps = [1 1; 0.3 3; 3 0.3; 0.3 3; 3 0.3];
muas = [0 0; 0 0.03; 0 0.03; 0.03 0; 0.03 0];
%musps = [0.3 3];
%muas = [0 0.03];

dz = 10;
n = 1.4;
bcs = 1;
scheme = 1;
verb = 0;    % whether to plot all signals.

% meas...
tmin = 500;
tmax = 5000;
dtgate = 100;
R = 40;

h = 0.5;       % adjust for certain accuracy.
beta = 0.05;

% set up resampled t array for matlab display...
ntgate=(tmax-tmin)/dtgate + 1;
sigt = tmin + dtgate*(0:ntgate)';
nt = length(sigt);
ns = size(musps,1);

% epsilon_box choices
es = 10.^(-7:0.25:0);
nes = length(es);
ee = 1e-15;   % "exact"

se = zeros(ns,1);
sa = zeros(ns,nes);
sl = zeros(ns,nes);
sigs = zeros(nes,ntgate);

% loop over systems...
for is = 1:ns
    
  musp = musps(is,:);
  mua = muas(is,:);

  disp(sprintf('SYS %d, EXACT: eps_a = %g...',is,ee));
  sige = slab(tmin,tmax,dtgate,R,mua,musp,dz,n,...
	      [ee,-h,-beta,ee,1e-12,-1,129,scheme,bcs]); %129
  se(is) = sige(ntgate);
  
  if 0
  % loop over a box sizes...
  for i=1:nes
    e = es(i);
    disp(sprintf('SYS %d, eps_a = %g...',is,e));
    sigs(i,:) = slab(tmin,tmax,dtgate,R,mua,musp,dz,n,...
	      [ee,-h,-beta,e,1e-12,-1,129,scheme,bcs])'; %129
    sa(is,i) = sigs(i,ntgate);
  end
  
  if verb
    figure;
    semilogy(sigt, sigs,'-');
    legnum(es, 2, '\epsilon_a=');
    hold on; semilogy(sigt, sige,'+'); hold off;
    title(sprintf('SYS %d: a box signals', is));
  end
  end
  
  % loop over L box sizes...
  for i=1:nes
    e = es(i);
    disp(sprintf('SYS %d, eps_L = %g...',is,e));
    sigs(i,:) = slab(tmin,tmax,dtgate,R,mua,musp,dz,n,...
	      [e,-h,-beta,ee,1e-12,-1,129,scheme,bcs])'; %129
    sl(is,i) = sigs(i,ntgate);
  end

  if verb
    figure;
    semilogy(sigt, sigs,'-');
    legnum(es, 2, '\epsilon_L=');
    hold on; semilogy(sigt, sige,'+'); hold off;
    title(sprintf('SYS %d: L box signals', is));
  end

end

% plot
ses = repmat(se, [1 nes]);
if verb
  figure;
  loglog(es, abs(sa-ses)./ses, '+-');
  legnum(1:ns, 1, 'SYS=');
  hold on; loglog(es, es, ':'); hold off;
  xlabel('\epsilon_{box}');
  title('a box');
  figure;
  loglog(es, abs(sl-ses)./ses, '+-');
  legnum(1:ns, 1, 'SYS=');
  hold on; loglog(es, es, ':'); hold off;
  xlabel('\epsilon_{box}');
  title('L box');
end

% fixed mu_shift results..
save boxdata_2.mat

return

% ----------------- plot, run separately:

load boxdata_2.mat
sesl = ses;   % exact values, when mu_shift fixed, used for L
sl_f = sl;

% load variable-mu_shift results for e_a convergence.
load boxdata.mat

% output figure...
locx = 0.55;
locy = 0.92;
fnts = 16;
figure;
subplot(1,2,1);
han = mmarker('loglog', es, abs(sa-ses)./ses, '-');
%typ = {'+b', '*g', 'dr', 'oc', 'xm'};
%for i=1:size(sa,1)
%  han(i) = loglog(es, abs(sa(i,:)-ses(i,:))./ses(i,:), [char(typ(i)) '-']);
%  hold on;
%end
%hold off;
%
%loglog(es, abs(sa-ses)./ses, '-+*');
%
set(gca,'FontSize',fnts);
legend(han, num2cellstr(1:ns, 2, 'SYS'), 2);  % upper left
hold on; loglog(es, es, ':'); hold off;
xlabel('\epsilon_{a}');
ylabel('relative error at t_{max}');
axis([1e-7 1 1e-11 1]);
text(locx,locy,'a)','units','normalized','fontsize',fnts);
subplot(1,2,2); han = mmarker('loglog', es, abs(sl_f-sesl)./sesl, '-');
set(gca,'FontSize',fnts);
legend(han, num2cellstr(1:ns, 2, 'SYS'), 2);  % upper left
hold on; loglog(es, es, ':'); hold off;
xlabel('\epsilon_{L}');
ylabel('relative error at t_{max}');
axis([1e-7 1 1e-11 1]);
text(locx,locy,'b)','units','normalized','fontsize',fnts);
print -depsc2 /home/alex/dot/mls/sla/fig/box_raw.eps
