% SLA figure conv of dt or beta.

% units mm, ps.
c = 0.2997;
mua = [0.01 0.01];
musp = [1.0 1.0];
dz = 10;
n = 1.4;
v = c/n;

% time-gates
tmin=100;  %100   % earlier = slower runtime of slab.c
tmax=3000;
dtgate=2.9;  % dtgate=100;
% set up resampled t array for matlab display...
ntgate=(tmax-tmin)/dtgate + 1;
sigt = tmin + dtgate*((1:ntgate)' - 1);

err_zbox = 1e-6;
err_h = 0.1;
err_lbox = 1e-6;
err_trunc = 1e-12;
scheme = 4;
  % CN var
bcs = 1;  % EBC

% error measure params... gamma, epsilon
g = 1e5;
e = 1e-2;

% sample radii, betas...
R = [0 20 40];
bs = [0.1 0.05 0.025];
%bs = [0.5 1.0 2.0 5.0 10.0];

nb = length(bs);
sigb = zeros(ntgate,length(R),nb);
for ib = 1:nb
  sigb(:,:,ib) = slab(tmin,tmax,dtgate,R,mua,musp,dz,n,...
		      [err_zbox,err_h,-bs(ib),err_lbox,err_trunc,...
		    -1,129,scheme,bcs]); %129
end

%sige = homog(sigt, mua(1), musp(1), v, R, bcs);
sige =  slab(tmin,tmax,dtgate,R,mua,musp,dz,n,...
	     [err_zbox,err_h,-0.5,err_lbox,err_trunc,...
	      -1,129,3,bcs]); % CN const.

peak = repmat(max(sige), [ntgate 1]);  % matrix for multiple R
errb = e*sige;  % fixed relative error size 
% poisson-noise type cutoff of dyn range...
errb = max(sqrt(g*sige./peak).*(peak/g), errb);
errb = max(peak/g, errb); %   hard dynamic range cutoff

figure;
lt = ['k: ';'k- ';'k--'];
for r=1:length(R)
  subplot(length(R),1,r);
  for n=1:nb
    if n==2
      hold on;
    end
    h = semilogy(sigt,abs(squeeze(sigb(:,r,n))-sige(:,r))./errb(:,r), lt(n,:));
  end
  hold off;
  %semilogy(repmat(sigt,[1 nb]),abs(squeeze(sigb(:,r,:))-...
%				   repmat(sige(:,r),[1 ...
 % nb]))./repmat(errb(:,r),[1 nb]),'k-');
  hline(1,'r-');
  set(gca,'FontSize',16);
  text(0.4,0.85,sprintf('R = %g mm',R(r)),'units','normalized','fontsize',16);
  axis([0 tmax 1e-2 1e2]);
  legnum(bs(1:nb), 2, '\beta=');
  if (r==length(R))
    xlabel('t (ps)');
  end
  ylabel('E(t)');
end

%print -depsc2 /home/alex/dot/mls/sla/fig/dtconv.eps


