% SLA paper, conv vs h, at fixed timestepping.
%
% based on TEST13 from test.m

% need to edit slab.c to use fixed h in setting dt_euler_kill_hf in
% calculate(), then recomplie with mex slab.c


% units mm, ps.
c = 0.2997;

% system
mua = [0.0 0.03];
musp =  [3.0 0.3];
dz = [10];
n = 1.4;
v = c/n;

% time-gates
tmin=200;  %100   % earlier = slower runtime of slab.c
tmax=3000;
dtgate=20;  % dtgate=100;
% set up resampled t array for matlab display...
ntgate=(tmax-tmin)/dtgate;
sigt = tmin + dtgate*(0:ntgate)';

% meas and errors
R = 20;
err_L = 1e-6;
err_dt = -0.05;
%err_h = -0.25;
err_R = 1e-6;
err_trunc = 1e-12;
verb = 129;
bcs = 0;
scheme = 4;
m = -1;    % radial mode to calc, eg 1, 5.

err_hs = -(0.5000:-0.05:0.100);   % negative err_h sets h directly
sigs = zeros(length(sigt),length(err_hs));
for i=1:length(err_hs)
  err_h = err_hs(i);
  sigs(:,i) = slab(tmin,tmax,dtgate,R,mua,musp,dz,n,...
		   [err_L,err_h,err_dt,err_R,err_trunc,m,verb,scheme,bcs,...
		   0,0.04]);  % override dt0 in const_var_cn.
end

% relative error convergence at 4 different times:
figure;
ts = [1 25 50 141];
sig_o = sigs(ts,length(err_hs));  % `converged' signal vals, ie smallest h.
quadr = inline('x(1)+x(2).*h.^2', 'x', 'h');
for i=1:4
  subplot(2,2,i);
  plot(-err_hs, sigs(ts(i),:)/sig_o(i) - 1, '.');
  x = lsqcurvefit(quadr, [0 1e-3], -err_hs, sigs(ts(i),:)/sig_o(i) - 1);
  xs = 0:0.01:-err_hs(1);  
  hold on; plot(xs, quadr(x, xs), '--'); hold off;
  xlabel('h (mm)');
  ylabel('relative signal');
  legend(sprintf('t = %.0f ps', sigt(ts(i))));
end
%title(sprintf('fig_hconv: mu_s''=[%g %g] dz=%g bcs=%d m=%d beta=%g',...
%	      musp(1), musp(2), dz, bcs, m, -err_dt));

%print -depsc2 /home/alex/dot/mls/sla/fig/hconv_1.eps
