% function [sig] = slab_homog_inf(tmin, tmax, dtgate, R, mua, musp,...
%                  dz {, n {, err}})
%
% Analytic version of slab.c for homogeneous semi-infinite case.
% Duplicates slab.c calling line. Raises error if mua, musp, dz do
% not correspond to semi-infinite homogeneous medium.
% err is ignored.
%
% See slab.m for further documentation on input arguments.

function [sig] = slab_homog_inf(tmin, tmax, dtgate, R, mua, musp,...
				dz, n, err)

if nargin==9
  %disp('exact solution: err will be ignored, apart from err(9).');
  if length(err)>8
    bcs = err(9);
  end
end
if length(mua)>1
  disp('mua must be single value!');
end
if length(musp)>1
  disp('musp must be single value!');
end
if length(musp)>1
  disp('musp must be single value!');
end
if ~isempty(dz)
  disp('dx must be [], corresponding to semi-infinite slab!');
end

% refractive index...
if nargin<=7
  n = 1.4;  % default in slab.c
end
c = 0.2997;
v = c/n;

% build timegates...
ntgate=(tmax-tmin)/dtgate + 1;
sigt = tmin + dtgate*((1:ntgate)' - 1);

% call analytic solution...
sig = homog(sigt, mua, musp, v, R, bcs);
