% testing slab.c
%
% barnett Dec 2002.

% units mm, ps.
c = 0.2997;

if ~exist('SYSTEM') 
    SYSTEM = input('SYSTEM (0=homog inf, 1=2layer) > ');  
end
if ~exist('TEST') 
    TEST = input(['TEST (0=1d lat, 1=1d conv, 2=3d sum modes, 3=3d,',...
		  '4=2d multi R, 5=2d multi e_trunc, 6=3d conv,',...
		  '7= , 8= , 9= , 10=probs SYS2, 11=dt_schemes, 12=timing,',...
		  '13=h-conv) > ']);  
end

switch SYSTEM
  
 case 0
  % semi-infinite homog
  mua = 0.01;
  musp = 1.0;
  dz = [];
  n = 1.4;    %  1.4
  v = c/n;

 case 1
  % semi-infinite 2-layer
  mua = [0.01 0.015];
%  musp = [0.8 1.5];
  musp = [1.0 1.5];
%  musp = [1.5 1];
  dz = [10];
  n = 1.4;
  v = c/n;

 case 2
  % caused blowup in fitting (binf)... now fixed.
  mua = [0.03 0.0];
  %musp =  [0.3 4.0];
  musp =  [3.0 0.3];
  dz = [10];
  n = 1.4;
  v = c/n;
  
 
 otherwise
  disp('unknown SYSTEM');
  
end
   
% time-gates
tmin=200;  %100   % earlier = slower runtime of slab.c
tmax=3000;
dtgate=20;  % dtgate=100;
% set up resampled t array for matlab display...
ntgate=(tmax-tmin)/dtgate + 1;
sigt = tmin + dtgate*((1:ntgate)' - 1);


switch TEST
  
 case 0
  % test against exact homog 1d solution (ZBC only), inc full lattice data.

  R = 0;
  err_zbox = 1e-6;
  err_h = 0.01;
  err_dt = 0.01;    % not used by forward euler
  
  sig = slab(tmin,tmax,dtgate,R,mua,musp,dz,n,[err_zbox,err_h,-err_dt,0,0,0,1]);
  
  dat = load('test.1d.dat');
  t = dat(:,1);
  u = dat(:,2);

  % exact solution for 1d (dirichlet)
  kap = 1/(3*musp);
  z0 = 1/musp;
  u_ex = z0 * exp(-z0*z0./(4*kap*v*t) - v*mua*t) ./ ...
	 (sqrt(4*pi)*(kap*v*t).^(3/2));
  t_ex = t;
  figure;subplot(2,1,1);semilogy(t,u,'+', t,u_ex,'-');
  subplot(2,1,2);loglog(t,abs(u./u_ex - 1),'+');
  title('0A: 1d unresampled surf signal (+) vs exact (-), and relative err');
  
  err = u - u_ex;
  ernm = dot(err',err)

  % load full lattice data, correct for exp decay...
  lat = load('test.1dlat.dat');
  nz = size(lat,2) - 1;
  h = lat(1,1); % lattice spacing
  lat = lat(:,2:nz+1); % clip off first column which was only used
                       % to get h 
  lat = lat .* repmat(exp(-v*mua*t), [1 nz]); % exp fac (mue_typ=mua)
  
  %figure; imagesc(log10(abs(lat))); C=caxis; colorbar; title('lat');

  % compute exact case...
  zs = (0:nz-1) * h;
  lt = length(t);
  zmat = kron(reshape(zs,[1 nz]), ones(lt ,1));
  tmat = kron(reshape(t,[lt 1]), ones(1, nz));
  lat_ex = exp(-(z0-zmat).^2./(4*kap*v*tmat)) -...
	   exp(-(z0+zmat).^2./(4*kap*v*tmat));
  % apply time-factors
  lat_ex = lat_ex .* repmat(exp(-v*mua*t)./sqrt(4*pi*kap*v*t), [1 nz]);

  %figure; imagesc(log10(abs(lat_ex))); caxis(C); colorbar; title('lat_ex');
  figure; imagesc(lat./lat_ex); caxis([0,2]); colorbar;
  title('0B: 1d fd lattice u(z,t)/u_{ex}(z,t), z (mm) across vs t (steps) down');
  
  % time slice to plot: crossover from FE to first C-N step.
  %i=30; % now can't predict which timestep this occurs at.
  %figure; plot(zs, lat(i,:), '+', zs, lat_ex(i,:), '-', ...
	%       zs, lat(i+1,:), '+', zs, lat_ex(i+1,:), '-');
  %title('0C: u(z,t) (+) and u_{ex}(z,t) (-), at end of FE and after first C-N');

  
 case 1
  % 1d plots, convergence with err_h to exact homog 1d solution (ZBC only)
  R = 0;
  err_zbox = 1e-6;
  
  % h size loop:
  err_hs = logspace(-1,-3,5);
  us = zeros(10000,length(err_hs));
  ts = zeros(size(us));
  u_exs = zeros(size(us));
  sigs = zeros(length(sigt),length(err_hs));
  for i=1:length(err_hs)
    err_h = err_hs(i);
    err_dt = 0.01;   % not used by forward euler, is by C-N.
    sigs(:,i) = slab(tmin,tmax,dtgate,R,mua,musp,dz,n,...
		     [err_zbox,err_h,err_dt,0,0,0,1,1,0]);
    dat = load('test.1d.dat');
    t = dat(:,1);
    ts(1:size(dat,1),i) = t;
    us(1:size(dat,1),i) = dat(:,2);
    kap = 1/(3*musp);
    z0 = 1/musp;
    u_exs(1:size(dat,1),i) = kap*z0 * exp(-z0*z0./(4*kap*v*t) - v*mua*t) ./ ...
	(sqrt(4*pi)*(kap*v*t).^(3/2));
    %  figure(2);subplot(2,1,1);
  %  hold on;semilogy(t,u,'+', t,u_ex,'-');hold off;
  %  subplot(2,1,2);hold on;plot(t,u./u_ex,'+');hold off;    
  end

  ir = 1:5; % i range
  % check error dies to what expect at tmin...
  figure;subplot(2,1,1);
  semilogy(ts(:,ir),us(:,ir),'+', ts(:,ir),u_exs(:,ir),'-');
  subplot(2,1,2);semilogy(ts(:,ir),abs(us(:,ir)./u_exs(:,ir)-1),'+');
  ax = axis;
  axis([0 100 ax(3) ax(4)]);
  title('1A: 1d convergence to exact, check t_min relative error');
  
  % check ratio error from 1...
  figure;loglog(ts,abs(us./u_exs-1),'+');
  title('1B: 1d fd, relative error convergence with err_h');
  
  % check the sampling routine...
  %figure;semilogy(ts,us,'+',  repmat(sigt,size(ir)),sigs,'-');

  % convergence of sampled to exact...
  t = sigt;
  sige = kap*z0 * exp(-z0*z0./(4*kap*v*t) - v*mua*t) ./ ...
	    (sqrt(4*pi)*(kap*v*t).^(3/2));
  figure;plot(sigt,sige,'-', repmat(sigt,size(ir)),sigs,'+');
  title('1C: 1d fd, relative error convergence with err_h');
  legnum([0 err_hs],4,'\epsilon_h,\epsilon_{dt} = ');
  
  
 case 2
  % 3d homog convergence with #modes, summing them, to exact, arbitrary R
  
  %R = 80;
  R = [80];
  N_R = length(R);
  err_zbox = 1e-6;
  err_h = 0.01; % 0.01; or -0.25 to set h directly
  err_dt = -1;    % not used by forward euler
  err_lbox = 1e-6;
  err_trunc = 1e-17;
  scheme = 3;
  
  N_m = 170; % can exceed N_m given by code - simply returns zeros
  
  sigs = zeros(length(sigt), N_m);
  for m=1:N_m
    sigs(:,m) = slab(tmin,tmax,dtgate,R,mua,musp,dz,n,...
		[err_zbox,err_h,err_dt,err_lbox,err_trunc,m,129*(m==1),scheme,1]);
  end
  
  if SYSTEM==0
    % convergence of sampled to exact 3d...
    sige = homog(sigt, mua, musp, v, R, 1);
    figure;
    semilogy(sigt,sige,'--', repmat(sigt,[1,N_m]),abs(sigs),'-',...
	     sigt,sum(sigs,2),'o');
    title('2A: 3d, contribs from modes, vs exact homog');
    
    %figure;loglog(sigt,abs(sum(sigs,2)./sige - 1), '+');
  else
    % general SYSTEM...
    figure;
    semilogy(repmat(sigt,[1,N_m]),abs(sigs),'+');
    hold on;
    ph = plot(sigt,abs(sum(sigs,2)),'k.');
    set(ph, 'MarkerSize', 15);
    title('2A: 3d, contribs from modes');
    hold off;
  end
  axis([0 tmax 1e-25 1e-5]);
    
  
  
 case 3
  % 3d homog compared to exact, for arbitrary multiple R
  
  R = 0:5:50;
  err_zbox = 1e-6;
  err_h = 0.01;
  err_dt = 0.01;    % not used by forward euler
  err_lbox = 1e-6;
  err_trunc = 1e-17;
  
  tic;
  sig = slab(tmin,tmax,dtgate,R,mua,musp,dz,n,...
	     [err_zbox,err_h,err_dt,err_lbox,err_trunc,-1,0,1,0]);
  toc
  
  % convergence of sampled to exact 3d...   ZBC
  sige = homog(sigt, mua, musp, v, R, 0);
  
  figure;
  semilogy(sigt,sig,'+');hold on;semilogy(sigt,sige,'-');
  semilogy(sigt,abs(sig-sige),'--');hold off;
  legnum(R,4,'R = ');
  title('3A: sig from C(+) vs exact (-) and abs err (--) for range of R');
  ax = axis;
  axis([ax(1) ax(2) 1e-23 ax(4)]);
  
  % show errors, only for values above a certain threshold...
  figure;
  loglog(sigt,abs((sig-sige)./sige).*(sig>4e-18),'+-');
  legnum(R,4,'R = ');
  title('3B: frac err for only sig>1e-17, for range of R');

  
  
 case 4
  % 2d (just summing bessels), convergence to exact, arbitrary multiple R
  
  R = 0:5:50;
  err_lbox = 1e-6;
  err_trunc = 1e-17;
  
  sig = slab(tmin,tmax,dtgate,R,mua,musp,dz,n,[0,0,0,err_lbox,err_trunc,-2,1]);
  sige = plane(sigt, mua, musp, v, R);
  
  figure;
  semilogy(sigt,sig,'+');hold on;semilogy(sigt,sige,'-');
  semilogy(sigt,abs(sig-sige),'--');hold off;
  legnum(R,4,'R = ');
  title('sig from C(+) vs exact (-) and abs err (--) for range of R');
  ax = axis;
  axis([ax(1) ax(2) 1e-23 ax(4)]);
  
  % show frac errors, only for values above a certain threshold...
  figure;
  loglog(sigt,abs((sig-sige)./sige).*(sig>1e-17),'+-');
  legnum(R,4,'R = ');
  title('frac err for only sig>1e-17, for range of R');
  
  % show errors...
  %figure;
  %loglog(sigt,abs(sig-sige),'+-');
  %legnum(R,4,'R = ');
  %title('abs err for range of R');
  
  % compare to uniform jitter at 1e-16 level
  %figure;
  %loglog(sigt,abs(sig./sige - 1),'+-');
  %hold on;loglog(sigt,(1e-16.*repmat(sige(:,1),[1 11]))./sige,'--');hold off;
  %hold on;loglog(sigt,1e-16./sige,'--');hold off;

  
 case 5
  % 2d (just summing bessels), multiple trunc errors, single R.
 
  R = 50;
  err_lbox = 1e-6;
  err_truncs = logspace(-17,-5,7);
  
  sigs = zeros(length(sigt),length(err_truncs));
  dats = cell([length(err_truncs) 1]); % different sizes so use cells
  for i=1:length(err_truncs)
    sigs(:,i) = slab(tmin,tmax,dtgate,R,mua,musp,dz,n,...
		     [0,0,0,err_lbox,err_truncs(i),-2,0]);
    dat = load('test.2d.dat')';
    dats{i} = dat;
  end
  sige = plane(sigt, mua, musp, v, R);
  siges = repmat(sige,[1 length(err_truncs)]);
  
  % show errors...
  figure;
  loglog(sigt,abs(sigs-siges),'+-')
  legnum(err_truncs,4,'\epsilon_{trunc} = ');
  hold on;loglog(sigt,sige,'--');hold off;
  title('abs errors, for variety of \epsilon_{trunc}s, cf exact sig');
  ax = axis;
  axis([ax(1) ax(2) 1e-30 ax(4)]);
  
  % compare summing in matlab, for err_trunc choice i...
  figure;
  i = 1;
  loglog(sigt,abs(dats{i}),'-', sigt,sum(dats{i},2),'o', ...
	 sigt,sigs(:,i),'+', sigt,sige,'--');
  ax = axis;
  axis([ax(1) ax(2) 1e-30 ax(4)]);
  title('Mode contribs to signal, for a single \epsilon_{trunc}');
  hold on;h=loglog(sigt,abs(dats{i}(:,1)/1e16),':');
  legend(h,'1e-16 * max term in sum');hold off;
  
  % compare summing in matlab, for err_trunc choice i...
  figure;
  h=loglog(sigt,abs(sum(dats{i},2)-sigs(:,i)),'-', sigt,abs(sigs(:,i)-sige),...
	 sigt,sigs(:,i),'+', sigt,sige,'--');
  legend(h, 'abs change due to summing in matlab vs C',...
	 'abs err C-summed vs exact', 'sig from C', 'sig exact');
  title('Summing mode contribs vs sig, for a single \epsilon_{trunc}');
  ax = axis;
  axis([ax(1) ax(2) 1e-30 ax(4)]);
  hold on;h=loglog(sigt,abs(dats{i}(:,1)/1e16),':');
  %legend(h,'1e-16 * max term in sum');    destroys other legends
  hold off;
  % conclusion: summing in matlab is 1e4 smaller errors than error
  % against exact (sige), when I had %.14g on j0.h data.
  % Now I have %.18g on j0.h data, difference is only 1e1.
  
    
 case 6
  % convergence in non-homogeneous medium
    
  R = 30;
  err_zbox = 1e-6;
  err_lbox = 1e-6;
  err_trunc = 1e-17;
  err_h = 0.01;
  errs = logspace(-1,-3,5);
  le = length(errs);
  
  sigs = zeros(length(sigt),le);
  for i=1:le
    err_dt = errs(i);
    disp(sprintf('err_dt = %g',err_dt))
    sigs(:,i) = slab(tmin,tmax,dtgate,R,mua,musp,dz,n,...
		     [err_zbox,err_h,err_dt,err_lbox,err_trunc,-1, ...
		      3]);
  end
  
  % convergence 
  figure;
  title('6A: convergence of full 3d with \epsilon_{dt}, const \epsilon_h');
  subplot(1,3,1);
  semilogy(sigt,sigs,'+-');
  legnum(errs,4,'\epsilon_{dt} = ');
  subplot(1,3,2);
  sigse = repmat(sigs(:,le),[1 le-1]); % use most accurate as `exact'.
  sigser = sigs(:,1:le-1) - sigse;
  loglog(errs(1:le-1), abs(sigser),'+-');
  subplot(1,3,3);
  loglog(errs(1:le-1), abs(sigser./sigse),'+-');
 
 
 case 7
  % not needed... can use.
  
  
  
  
 case 8
  % single mode full 1d lattice data image, any system.
  
  R = 25;
  err_zbox = 1e-6;
  err_h = 0.01;
  err_dt = -0.05;
  err_lbox = 1e-6;
  err_trunc = 1e-17;
  
  m=1;  % mode
  sig = slab(tmin,tmax,dtgate,R,mua,musp,dz,n,[err_zbox,err_h,err_dt,...
		    err_lbox,err_trunc,m,385]); % verb flags test.1d.dat
  dat = load('test.1d.dat');
  t = dat(:,1);
  u = dat(:,2);
  lat = load('test.1dlat.dat');
  nz = size(lat,2) - 1;
  h = lat(1,1); % lattice spacing
  lat = lat(:,2:nz+1); % clip off first column which was only used
                       % to get h
  mu = mua(length(mua));
  %lat = lat .* repmat(exp(-v*mu*t), [1 nz]); % exp fac (mue_typ=mu)
  
  figure; imagesc(log10(abs(lat))); C=caxis; colorbar;
  title('8A: lat, without mue_typ exp decay');

 case 9
  % compare analytic BC choices against each other. (SYSTEM=0)

  sigt = logspace(0, 4, 1000);    % new times
  
  R = [5 7 10 15 20 30 50];
  
  z0 = 1/musp;
  zb = 1.96/musp;  % n=1.4 only
  % sig = ((z0+zb)/z0)*homog(sigt, mua, musp, v, R, 0); % scale for BCS 1 
  sig = homog(sigt, mua, musp, v, R, 1); 
  sigebc = homog(sigt, mua, musp, v, R, 2);  % 1 or 2
  
  figure;
  maxsigebc = repmat(kron(max(sigebc), ones(length(sigt), 1)),[1 2]);
  plot(sigt, [sigebc sig]./maxsigebc, '-');
  legnum(R, 2, 'R=');
  title('homog.m: case 0 cf case 1 (scaled to case 1 has max of 1)');
  
  figure;
  maxsigs = kron(max([sigebc sig]), ones(length(sigt), 1));
  plot(sigt, [sigebc sig]./maxsigs, '-');
  legnum(R, 2, 'R=');
  title('homog.m: case 0 cf case 1 (scaled so all have max of 1)');
  
  figure;
  loglog(sigt, sig, ':'); 
  hold on; loglog(sigt, sigebc, '-'); legnum(R, 2, 'R='); hold off;
  axis([min(sigt) max(sigt) max(max(sig))/1e10 max(max(sig))]);
  title('homog.m: case 0 (Dir) dotted, case 1 (EBC) solid');
  
  figure;
  loglog(sigt, (sig-sigebc)./sigebc, '-');
  legnum(R, 2, 'R=');
  title('homog.m: rel err of case 0 cf case 1');
  
 case 10
  % diagnose problems in SYSTEM 2 (similar to case 2)
  
  R = 60;
  err_zbox = 1e-6;
  err_h = 0.03;
  err_dt = 0.01;    % not used by forward euler
  err_lbox = 1e-6;
  err_trunc = 1e-12;
  
  N_m = 100; % can exceed N_m given by code - simply returns zeros
  
  if 1
  sigs = zeros(length(sigt),N_m);
  for m=1:N_m
    sigs(:,m) = slab(tmin,tmax,dtgate,R,mua,musp,dz,n,...
		     [err_zbox,err_h,err_dt,err_lbox,err_trunc,m,3*(m==1)]);
		    % [err_zbox,err_h,err_dt,err_lbox,err_trunc,m,3*(m==1)]);
  end
  end
  
  % 1d lattice for single mode:
  m = 60;
  % -err_dt makes output 1dlat data
  sig = slab(tmin,tmax,dtgate,R,mua,musp,dz,n,...
		     [err_zbox,err_h,-err_dt,err_lbox,err_trunc,m,0]);
  
  %figure;
  %semilogy(sigt, abs(sig), '+');
  %title('10A: single mode contrib, with mue_{typ} exp decay');
  
  % read 1dlat data
  lat = load('test.1dlat.dat');
  nz = size(lat,2) - 1;
  h = lat(1,1); % lattice spacing
  lat = lat(:,2:nz+1); % clip off first column which was only used
                       % to get h
  
  figure; imagesc(log10(abs(lat))); C=caxis; colorbar;
  title('10B: lat, without mue_{typ} exp decay');

  if 1
  figure;
  semilogy(repmat(sigt,[1,N_m]),abs(sigs),'+');
  hold on;
  ph = plot(sigt,sum(sigs,2),'k.');
  set(ph, 'MarkerSize', 15);
  title('10C: 3d, contribs from modes');
  hold off;
  end
  
 case 11
  % 3d convergence with #modes, summing them. Different t-step schemes.
  
  %R = 80;
  R = [40];
  N_R = length(R);
  err_zbox = 1e-6;
  err_h = 0.03;
  err_dt = 0.01;    % not used by forward euler
  err_lbox = 1e-6;
  err_trunc = 1e-12;
  
  N_m = 100; % can exceed N_m given by code - simply returns zeros
  
  sigs_be = zeros(length(sigt), N_m);
  sigs_cn = zeros(length(sigt), N_m);
  sigs_fe = zeros(length(sigt), N_m);
  for m=1:N_m
    sigs_be(:,m) = slab(tmin,tmax,dtgate,R,mua,musp,dz,n,...
		     [err_zbox,err_h,err_dt,err_lbox,err_trunc,m,3*(m==1),0]);
    sigs_cn(:,m) = slab(tmin,tmax,dtgate,R,mua,musp,dz,n,...
		     [err_zbox,err_h,err_dt,err_lbox,err_trunc,m,3*(m==1),1]);
    sigs_fe(:,m) = slab(tmin,tmax,dtgate,R,mua,musp,dz,n,...
		     [err_zbox,err_h,err_dt,err_lbox,err_trunc,m,3*(m==1),2]);
  end
  sig_fe=sum(sigs_fe,2);
  sig_be=sum(sigs_be,2);
  sig_cn=sum(sigs_cn,2);
  
  % general SYSTEM...
  if 1
  figure;
  semilogy(repmat(sigt,[1,N_m]),abs(sigs_be),'+');
  hold on;
  semilogy(repmat(sigt,[1,N_m]),abs(sigs_cn),'.');
  semilogy(repmat(sigt,[1,N_m]),abs(sigs_fe),'*');
  ph = plot(sigt,sig_be,'k.');
  set(ph, 'MarkerSize', 25);
  ph = plot(sigt,sig_cn,'b.');
  set(ph, 'MarkerSize', 25);
  ph = plot(sigt,sig_fe,'r.');
  set(ph, 'MarkerSize', 25);
  title(['11A: 3d, contribs from modes. t-step schemes 0=BE(+k)' ...
	 ' 1=CNvar(.b) 2=FE(*r)']);
  hold off;
  end
  
  % taking FE as standard, compare CNvar errs relative to max signal.
  figure;
  maxs = max(sig_fe);
  subplot(2,1,1); semilogy(sigt,sig_fe/maxs,'-+');
  subplot(2,1,2); plot(sigt,(sig_cn-sig_fe)/maxs,'-+');
  title('11B: FE signal and CNvar-FE errs, rel to peak signal');
  
  
 case 12
  % timing for dt schemes, different tmin choices.
  
  R = [40];
  err_zbox = 1e-6;
  err_h = 0.01;
  err_dt = 0.003;    % not used by forward euler
  err_lbox = 1e-6;
  err_trunc = 1e-12;
  
  tmins=[50 100 200 500];
  tmax=3000;
  dtgate=25;  % dtgate=100;
  
  for tmin=tmins
    disp(sprintf('\ntmin = %f',tmin));
    ntgate=(tmax-tmin)/dtgate;
    sigt = tmin + dtgate*((1:ntgate)' - 1);
    
    % verb codes can be 129,128,128 for the following... nice output
    tic;
    sig_be = slab(tmin,tmax,dtgate,R,mua,musp,dz,n,...
		  [err_zbox,err_h,err_dt,err_lbox,err_trunc,-1,0,0,0]);
    disp(sprintf('0:BE \t%f sec',toc));
    tic;
    sig_cn = slab(tmin,tmax,dtgate,R,mua,musp,dz,n,...
		  [err_zbox,err_h,err_dt,err_lbox,err_trunc,-1,0,1,0]);
    disp(sprintf('1:CNvar \t%f sec',toc));
    tic;
    sig_fe = slab(tmin,tmax,dtgate,R,mua,musp,dz,n,...
		  [err_zbox,err_h,err_dt,err_lbox,err_trunc,-1,0,2,0]);
    disp(sprintf('2:FE \t%f sec',toc));
  end
  
 case 13
  % convergence with h, for 1d system......................................
  R = 0;
  err_L = 1e-6;
  err_dt = -0.05;
  %err_h = -0.25;
  err_R = 1e-6;
  err_trunc = 1e-12;
  verb = 0;
  bcs = 0;
  m = -1;    % radial mode to calc, eg 1, 5.
  
  % h size loop:
  %err_hs = -(0.2500:-0.0003:0.2200);
  err_hs = -(0.5000:-0.005:0.100);
  sigs = zeros(length(sigt),length(err_hs));
  for i=1:length(err_hs)
    err_h = err_hs(i);
    sigs(:,i) = slab(tmin,tmax,dtgate,R,mua,musp,dz,n,...
		     [err_L,err_h,err_dt,err_R,err_trunc,m,verb,1,bcs]);
  end

  % relative convergence at different times:
  figure;
  ts = [1 25 50 140];
  sig_o = sigs(ts,length(err_hs));
  for i=1:4
    subplot(2,2,i);
    plot(-err_hs, sigs(ts(i),:)/sig_o(i) - 1, '.');
    xlabel('h');
    ylabel('det sig');
    legend(sprintf('t = %.0f ps', sigt(ts(i))));
  end
  if SYSTEM==0
    title(sprintf('TEST13: mu_s''=%g bcs=%d m=%d beta=%g',...
		  musp, bcs, m, -err_dt));
  else
    title(sprintf('TEST13: mu_s''=[%g %g] dz=%g bcs=%d m=%d beta=%g',...
		  musp(1), musp(2), dz, bcs, m, -err_dt));
  end
  
  
  
  
 otherwise
  disp('unknown TEST');
  
  
end

%end
