%{
#include <stdlib.h>
#include "parser.tab.h"

extern void yyerror(const char *);

int linenumber = 0;
%}

	/* Characters allowed in unquoted filenames */
FILENAME [-_#.+a-zA-Z0-9/]

%option caseless
%option noyywrap

%s INQ ENDQ
%%
                        /* Tokens, most of which can be abreviated */
nphotons                return NPHOTONS;
seed                    return SEED;
seg[mentation_file]*    return SEGFILE;
freq                    return FREQUENCY;
c0                      return CLIGHT;

start_?time             return STIME;
gate_?width             return DTIME;
ngates?                 return NTIME;

tissue                  return TISSUETYPE;

mus                     return MUS;
mua                     return MUA;
g                       return G;
index                   return N;
n                       return N;

det                     return DETECTOR;
detector                return DETECTOR;
src                     return SOURCE;
source                  return SOURCE;

pos                     return SDPOS;
position                return SDPOS;
dir                     return SDDIR;
direction               return SDDIR;
rad                     return SDRAD;
radius                  return SDRAD;
na                      return SDNA;

img_?x                  return IMGX;
img_?y                  return IMGY;
img_?z                  return IMGZ;
image_?x                return IMGX;
image_?y                return IMGY;
image_?z                return IMGZ;

dx                      return DX;
dy                      return DY;
dz                      return DZ;

nxvox                   return NXSTEP;
nyvox                   return NYSTEP;
nzvox                   return NZSTEP;

			/* Quoted filenames must start with a normal */
			/*  FILENAME character but may contain embedded */
			/*  special characters (space, etc.) after that */
\"/{FILENAME}           BEGIN(INQ);
<INQ>.*/\"              { yylval.string = yytext; BEGIN(ENDQ);
                          return FILENAME; }
<ENDQ>\"                BEGIN(INITIAL);

			/* Primitives */
[-+0-9]+                { yylval.integer = atoi(yytext); return DECNUMBER; }
[-+.eE0-9]+             { yylval.real    = atof(yytext); return REALNUMBER; }
{FILENAME}+             { yylval.string  = yytext;       return FILENAME; }

"["   return '[';	/* begin vector */
"]"   return ']';	/* end   vector */
"{"   return '{';	/* begin grouping */
"}"   return '}';	/* end   grouping */

,                       /* IGNORE: optional separator */
=                       /* IGNORE: optional assignment */
;.*                     /* IGNORE: comment */
[ \t]+                  /* IGNORE: whitespace */
\r?\n                   linenumber++;     /* End of line */
<*><<EOF>>              return 0;         /* Done, returns zero */
.                       yyerror("Unrecognized token");

%%

void parsefile(FILE *fp)
{
  extern int yyparse(void *);

  /* Reset the parser with a new file pointer */
  yyrestart(fp);

  /* Parse this file */
  yyparse(NULL);

  /* Done */
  return;
}
