% function [phys, x_f] = phys_slab(mua, musp, dz, R {, bcs {, n}})
%
% build phys struct for model 2.
% Also returns z_f, the full z vector corresponding to input values.
%
% barnett 1/29/03
% 5/28/03 added refractive index field in phys.

function [phys, z_f] = phys_slab(mua, musp, dz, R, bcs, n)

phys.model = 2;
phys.Ndz = length(dz);
phys.Nl = length(mua);
if length(musp)~=phys.Nl
  disp('mua and musp must have same length!');
end
phys.Ns = 1;
phys.Nd = length(R);
z_f = [mua musp dz R];
phys.precon_z = [100*ones(size(mua)) ones(size(musp)) 0.2*ones(size(dz)) ...
		   0.1*ones(size(R))];
phys.names_z = [num2cellstr(1:phys.Nl, 1, 'mua')' ...
	   num2cellstr(1:phys.Nl, 1, 'musp')' ...
	 num2cellstr(1:phys.Ndz, 1, 'dz')' num2cellstr(1:phys.Nd, 2, 'R')'];
phys.units_z = [cellstr(repmat('1/mm',[2*phys.Nl 1]))' ...
		cellstr(repmat('mm',[phys.Ndz+phys.Nd 1]))'];

% where to begin optim, in preconditioned units
phys.start_z = [1*ones(size(mua)) 2*ones(size(musp)) 1*ones(size(dz)) ...
		1*ones(size(R))];

% prior model... unpreconditioned ie in physical units
phys.prior.LB = [0*ones(size(mua))   0.3*ones(size(musp)) ...
		 1*ones(size(dz)) 3*ones(size(R)) ];
phys.prior.UB = [0.1*ones(size(mua)) 4*ones(size(musp)) ...
		 50*ones(size(dz))  100*ones(size(R)) ];

if nargin>4
  phys.bcs = bcs;
else
  phys.bcs = 1;  % default BCs are EBC.
end

if nargin>5
  phys.n = n;
else
  phys.n = 1.4;  % default refractive index
end


% end
