% function [phys, x_f] = phys_slab_homog_inf(mua, musp, R {, bcs {,n}})
%
% build phys struct for model 1, including any prior info.
% Also returns z_f, the full z vector corresponding to input values.
%
% barnett 1/29/03
% 5/28/03 added refractive index, bcs field to phys.

function [phys, z_f] = phys_slab_homog_inf(mua, musp, R, bcs, n)

phys.model = 1;
phys.Nl = 1;
if length(mua)~=1 | length(musp)~=1
  disp('phys_slab_homog_inf: mua and musp must be single numbers!');
end
phys.Ns = 1;
phys.Nd = length(R);
z_f = [mua musp R];
phys.precon_z = [100 1 0.1*ones(size(R))];
phys.names_z = [{'mua'} {'musp'} num2cellstr(1:phys.Nd, 2, 'R')'];
phys.units_z = [{'1/mm'} {'1/mm'} cellstr(repmat('mm',[phys.Nd 1]))'];

% where to begin optim, in preconditioned units
phys.start_z = [1 2 2*ones(size(R))];

% prior model... unpreconditioned ie in physical units
phys.prior.LB = [0   0.3 3*ones(size(R))  ];
phys.prior.UB = [0.1 4   100*ones(size(R))];

if nargin>3
  phys.bcs = bcs;
else
  phys.bcs = 1;  % default BCs are EBC.
end

if nargin>4
  phys.n = n;
else
  phys.n = 1.4;  % default refractive index
end

% end
