% function [phys, x_f] = phys_tFDimg(mua, musp, name {, bcs {,n}})
%
% build phys struct for model 3. Just mua, musp as variable params.
% CSF properties are inserted here, as part of phys struct.
% Timegate info is controlled as in models 1,2 from phys struct.
% (Future: include source & det locations, within some plane
% defined in geom file).
%
% Returns z_f, the full z vector corresponding to input values.
%
% barnett 5/28/03

function [phys, z_f] = phys_tFDimg(mua, musp, name, bcs, n)

phys.model = 3;
phys.name = name;
sys = load_cfg(name);
phys.Nl = length(mua);  % cannot exceed max val in .seg file
if length(musp)~=phys.Nl
  disp('phys_tFDimg: mua and musp must have same length!');
end
phys.Ns = sys.nsrc;
phys.Nd = sys.ndet;
if sys.ntissue>0
  if sys.ntissue~=phys.Nl
    disp('phys_tFDimg: ntissue doesnt match # of layers from mua, musp!');
  end
end
phys.len_sc = sys.xrange(1) / 2.0;    % dx / standard lattice size (2mm).

z_f = [mua musp];
phys.precon_z = [100*ones(size(mua)) ones(size(musp))];
phys.names_z = [num2cellstr(1:phys.Nl, 1, 'mua')' ...
	   num2cellstr(1:phys.Nl, 1, 'musp')'];
phys.units_z = [cellstr(repmat('1/mm',[2*phys.Nl 1]))'];

% where to begin optim, in preconditioned units
phys.start_z = [1*ones(size(mua)) 2*ones(size(musp))];

% prior model... unpreconditioned ie in physical units
phys.prior.LB = [0*ones(size(mua))   0.4*ones(size(musp))];
phys.prior.UB = [0.1*ones(size(mua)) 4*ones(size(musp))];

if nargin>3
  phys.bcs = bcs;
else
  phys.bcs = 1;  % default BCs are EBC.
end

if nargin>4
  phys.n = n;
else
  phys.n = 1.4;  % default refractive index
end

% CSF handling...
phys.CSF.mua = 0.0004;   % in physical units 1/mm
phys.CSF.musp = 0.4;     % ditto
%phys.CSF.mua = 0.0178;   % same as brain
%phys.CSF.musp = 1.25;     % ditto

% end
