/* Resample a segmentation file of xyz ordering, optionally flip x-y coords.
 * 
 * barnett 3/22/02
 */

#include <stdio.h>
#include <string.h>

int main (int argc, char *argv[])
{
  int x,y,z,i,j,k,nx,ny,nz,n,mx,my,mz,m;
  int xl,xh,yl,yh,zl,zh,zero_it;
  int zero_faces=0, zero_faces_final=0, flip=0, chop=0; /* cmd line defaults */
  double rx,ry,rz;
  FILE *fp;
  char in[256], out[256], *opts;
  char *data, tt;


  /* parse command line... */
  if (argc==1) {
    printf("usage: reseg nx ny nz rx ry rz [opts [params]] infilename outfilename\n\n");
    printf("       nx,ny,nz = original size; rx,ry,rz = ratio to expand by.\n");
    printf("       opts:\n       z - set box faces to tissue 0 (air), before any rescale\n");
    printf("       Z - set final box faces to tissue 0, after rescale\n");
    printf("       f - flips x and y to convert to Matlab ordering.\n");
    printf("       c xl xh yl yh zl zy - chops all but cuboid defined in final\n (resampled) voxel units, with 0 as origin.\n");
    exit(1);
  }
  i = 1;   /* moving pointer to arguments */
  sscanf(argv[i++],"%d",&nx);
  sscanf(argv[i++],"%d",&ny);
  sscanf(argv[i++],"%d",&nz);
  sscanf(argv[i++],"%lf",&rx);
  sscanf(argv[i++],"%lf",&ry);
  sscanf(argv[i++],"%lf",&rz);
  if (argc>=10) {
    /* there is an opts string... */
    opts = argv[i++];
    if (strchr(opts,'f')!=NULL) {
      flip = 1;
      printf("Will flip x-y.\n");
    }
    if (strchr(opts,'z')!=NULL) {
      zero_faces = 1;
      printf("Will zero box faces before.\n");
    }
    if (strchr(opts,'Z')!=NULL) {
      zero_faces_final = 1;
      printf("Will zero final box faces.\n");
    }
    if (strchr(opts,'c')!=NULL) {
      chop = 1;
      printf("Will chop off all but remainding cuboid.\n");
      sscanf(argv[i++],"%d",&xl);
      sscanf(argv[i++],"%d",&xh);
      sscanf(argv[i++],"%d",&yl);
      sscanf(argv[i++],"%d",&yh);
      sscanf(argv[i++],"%d",&zl);
      sscanf(argv[i++],"%d",&zh);
    }
  }
  sscanf(argv[i++],"%s",in);
  sscanf(argv[i++],"%s",out);

  /* read in segfile... */
  n = nx*ny*nz;
  data = (char*)malloc(n*sizeof(char));
  if (data==NULL) {
    printf("failed to malloc %d bytes.\n",n*sizeof(char));
    exit(1);
  }

  if ((fp = fopen(in,"rb"))==NULL) {
    printf("can't open %s for reading.\n",in);
    exit(1);
  }
  if (fread(data,sizeof(char),n,fp)!=n) {
    printf("file too short for size %d given.\n",n);
    exit(1);
  }
  fclose(fp);
  printf("%s read.\n",in);

  /* zero faces... done before rescale */
  if (zero_faces) {
    for (i=0; i<nx; ++i)
      for (j=0; j<ny; ++j) {
	data[j*nx + i] = (char)0;
	data[(nz-1)*nx*ny + j*nx + i] = (char)0;
      }
    for (j=0; j<ny; ++j)
      for (k=0; k<nz; ++k) {
	data[k*ny*nx + j*nx] = (char)0;
	data[k*nx*ny + j*nx + (nx-1)] = (char)0;
      }
    for (k=0; k<nz; ++k)
      for (i=0; i<nx; ++i) {
	data[k*ny*nx + i] = (char)0;
	data[k*ny*nx + (ny-1)*nx + i] = (char)0;
      }
  }

  /* compute rescaled sizes... */
  mx = (int)(rx*nx + 0.5);
  my = (int)(ry*ny + 0.5);
  mz = (int)(rz*nz + 0.5);
  printf("\nRescaled xyz size is:  %d %d %d\n",mx,my,mz);
  if (chop) {
    /* check if requested chopped block is inside mx,my,mz... */
    printf("Chopped region is:  %d %d %d\n",xh-xl+1,yh-yl+1,zh-zl+1);
  } else {
    /* make chopping cuboid the whole thing... */
    xl = 0;
    xh = mx-1;
    yl = 0;
    yh = my-1;
    zl = 0;
    zh = mz-1;
  }

  /* output as rescaled array... */
  if ((fp = fopen(out,"wb"))==NULL) {
    printf("can't open %s for writing.\n",in);
    exit(1);
  }
  for (k=zl;k<=zh;++k) {
    z = (int)((k + 0.5)/rz);
    if (flip) {
      for (i=xl;i<=xh;++i) {
	x = (int)((i + 0.5)/rx);
	for (j=yl;j<=yh;++j) {
	  y = (int)((j + 0.5)/ry);
	  if (x>=nx || y>=ny || z>=nz)
	    printf("%d %d %d      %d %d %d\n",i,j,k,x,y,z);
	  zero_it = zero_faces_final && (k==zl || k==zh || j==yl || j==yh ||
					 i==xl || i==xh);
	  tt = zero_it ? (char)0 : data[z*nx*ny + y*nx + x];
	  if (fwrite(&tt, sizeof(char), 1, fp)!=1)
	    printf("error writing at xyz output %d %d %d.\n",x,y,z);
	}
      }
    } else {
      for (j=yl;j<=yh;++j) {
	y = (int)((j + 0.5)/ry);
	for (i=xl;i<=xh;++i) {
	  x = (int)((i + 0.5)/rx);
	  if (x>=nx || y>=ny || z>=nz)
	    printf("%d %d %d      %d %d %d\n",i,j,k,x,y,z);
	  zero_it = zero_faces_final && (k==zl || k==zh || j==yl || j==yh ||
					 i==xl || i==xh);
	  tt = zero_it ? (char)0 : data[z*nx*ny + y*nx + x];
	  if (fwrite(&tt, sizeof(char), 1, fp)!=1)
	    printf("error writing at xyz output %d %d %d.\n",x,y,z);
	}
      }
    }
  }
  fclose(fp);

  return 0;
}
